package me.wazup.hideandseek.utils;

import org.bukkit.Material;
import org.bukkit.inventory.InventoryHolder;
import org.bukkit.Bukkit;
import me.wazup.hideandseek.Customization;
import org.bukkit.inventory.Inventory;
import org.bukkit.entity.Player;
import java.util.Collection;
import java.util.List;
import org.bukkit.ChatColor;
import java.util.Iterator;
import org.bukkit.inventory.ItemStack;
import java.util.ArrayList;
import java.util.HashMap;

public class FlexibleSmartInventory
{
    private final String mainName;
    private HashMap<Integer, ArrayList<CustomInventory>> inventories;
    
    public FlexibleSmartInventory(final String mainName) {
        this.inventories = new HashMap<Integer, ArrayList<CustomInventory>>();
        this.mainName = mainName;
    }
    
    public FlexibleSmartInventory clone() {
        final FlexibleSmartInventory flexibleSmartInventory = new FlexibleSmartInventory(this.mainName);
        flexibleSmartInventory.inventories = (HashMap)this.inventories.clone();
        return flexibleSmartInventory;
    }
    
    public int addInventory(final String s, final int n, final HashMap<Integer, ItemStack> hashMap) {
        InventorySize inventorySize = InventorySize.SMALL;
        if (inventorySize.size < n) {
            inventorySize = InventorySize.MEDIUM;
        }
        if (inventorySize.size < n) {
            inventorySize = InventorySize.LARGE;
        }
        if (inventorySize.size < n) {
            inventorySize = InventorySize.MAX;
        }
        return this.addInventory(s, inventorySize, hashMap);
    }
    
    public int addInventory(final String str, final InventorySize inventorySize, final HashMap<Integer, ItemStack> hashMap) {
        final int size = this.inventories.size();
        final CustomInventory e = new CustomInventory(this.mainName + (str.isEmpty() ? "" : (": " + str)), inventorySize, hashMap, size, 0);
        final ArrayList<CustomInventory> value = new ArrayList<CustomInventory>();
        value.add(e);
        if (size > 0) {
            e.addNavigationButtons(false, true);
            this.getLastInventory(size - 1).addNavigationButtons(true, false);
        }
        this.inventories.put(size, value);
        return size;
    }
    
    private CustomInventory getLastInventory(final int i) {
        final ArrayList list = this.inventories.get(i);
        return (CustomInventory)list.get(list.size() - 1);
    }
    
    public ArrayList<PositionData> prepareForItems(final int i, final int n) {
        final ArrayList list = new ArrayList();
        final ArrayList list2 = this.inventories.get(i);
        int n2 = n;
        final HashMap hashMap = new HashMap();
        for (final CustomInventory customInventory : list2) {
            ArrayList<Integer> value;
            for (value = customInventory.getEmptySlots(); n2 > value.size() && !customInventory.size.equals(InventorySize.MAX); value = customInventory.getEmptySlots()) {
                customInventory.expand();
            }
            hashMap.put(customInventory.subId, value);
            n2 -= value.size();
            if (n2 < 1) {
                break;
            }
        }
        if (n2 > 0) {
            this.getLastInventory(i).addInventory(InventorySize.SMALL, this.inventories);
            return this.prepareForItems(i, n);
        }
        for (final int intValue : hashMap.keySet()) {
            final Iterator iterator3 = hashMap.get(intValue).iterator();
            while (iterator3.hasNext()) {
                list.add(new PositionData(i, intValue, (int)iterator3.next()));
            }
        }
        return list;
    }
    
    public void removeItem(final int i, final String anObject) {
        for (final CustomInventory customInventory : this.inventories.get(i)) {
            for (final ItemStack itemStack : customInventory.getContents()) {
                if (ChatColor.stripColor(itemStack.getItemMeta().getDisplayName()).equals(anObject)) {
                    customInventory.removeItem(itemStack);
                    break;
                }
            }
        }
    }
    
    public List<ItemStack> getContents() {
        final ArrayList list = new ArrayList();
        for (int i = 0; i < this.inventories.size(); ++i) {
            final Iterator iterator = this.inventories.get(i).iterator();
            while (iterator.hasNext()) {
                list.addAll(((CustomInventory)iterator.next()).getContents());
            }
        }
        return list;
    }
    
    public List<ItemStack> getContents(final int i) {
        final ArrayList list = new ArrayList();
        final Iterator iterator = this.inventories.get(i).iterator();
        while (iterator.hasNext()) {
            list.addAll(((CustomInventory)iterator.next()).getContents());
        }
        return list;
    }
    
    public void setItem(final int i, final int index, final int n, final ItemStack itemStack) {
        this.inventories.get(i).get(index).setItem(n, itemStack);
    }
    
    public PositionData addItem(final ItemStack itemStack, final int i) {
        return this.getLastInventory(i).addItem(itemStack, i, this.inventories.get(i).size() - 1, this.inventories);
    }
    
    public ItemStack getItem(final int i, final int index, final int n) {
        return this.inventories.get(i).get(index).inventory.getItem(n);
    }
    
    public void open(final Player player) {
        this.open(player, 0, 0);
    }
    
    public void open(final Player player, final int i, final int index) {
        this.inventories.get(i).get(index).open(player);
    }
    
    public boolean handleScroll(final Player player, final ItemStack itemStack, final Inventory obj) {
        if (!Utils.compareItem(itemStack, Customization.getInstance().items.get("Next")) && !Utils.compareItem(itemStack, Customization.getInstance().items.get("Previous"))) {
            return false;
        }
        for (int i = 0; i < this.inventories.size(); ++i) {
            final List list = this.inventories.get(i);
            for (final CustomInventory customInventory : list) {
                if (customInventory.inventory.equals(obj)) {
                    int n;
                    int n2;
                    if (Utils.compareItem(itemStack, Customization.getInstance().items.get("Next"))) {
                        if (customInventory.subId + 1 < list.size()) {
                            n = customInventory.id;
                            n2 = customInventory.subId + 1;
                        }
                        else {
                            n = customInventory.id + 1;
                            n2 = 0;
                        }
                    }
                    else if (customInventory.subId == 0) {
                        n = customInventory.id - 1;
                        n2 = this.inventories.get(i - 1).size() - 1;
                    }
                    else {
                        n = customInventory.id;
                        n2 = customInventory.subId - 1;
                    }
                    this.open(player, n, n2);
                    break;
                }
            }
        }
        return true;
    }
    
    public String getName() {
        return this.mainName;
    }
    
    public enum InventorySize
    {
        SMALL(27), 
        MEDIUM(36), 
        LARGE(45), 
        MAX(54);
        
        int size;
        
        private InventorySize(final int size) {
            this.size = size;
        }
    }
    
    public class PositionData
    {
        public int id;
        public int subId;
        public int slot;
        
        public PositionData(final int id, final int subId, final int slot) {
            this.id = id;
            this.subId = subId;
            this.slot = slot;
        }
    }
    
    public class CustomInventory
    {
        int id;
        int subId;
        boolean hasNextButton;
        boolean hasBackButton;
        String title;
        Inventory inventory;
        ArrayList<Integer> slots;
        InventorySize size;
        HashMap<Integer, ItemStack> specialItems;
        
        public CustomInventory(final String s, final InventorySize inventorySize, final HashMap<Integer, ItemStack> hashMap, final int n, final int n2) {
            this.setupCustomInventory(s, inventorySize, hashMap, n, n2);
        }
        
        public void setupCustomInventory(final String title, final InventorySize size, final HashMap<Integer, ItemStack> specialItems, final int id, final int subId) {
            this.title = title;
            this.size = size;
            this.specialItems = specialItems;
            this.id = id;
            this.subId = subId;
            Utils.cageInventory(this.inventory = Bukkit.createInventory((InventoryHolder)null, size.size, title), false);
            if (specialItems != null) {
                for (final int intValue : specialItems.keySet()) {
                    this.inventory.setItem(intValue + this.inventory.getSize() - 9, (ItemStack)specialItems.get(intValue));
                }
            }
            this.slots = new ArrayList<Integer>();
            for (int i = 0; i < this.inventory.getSize(); ++i) {
                if (this.inventory.getItem(i) == null) {
                    this.slots.add(i);
                }
            }
        }
        
        public void open(final Player player) {
            player.openInventory(this.inventory);
        }
        
        public void addNavigationButtons(final boolean hasNextButton, final boolean hasBackButton) {
            this.hasNextButton = hasNextButton;
            this.hasBackButton = hasBackButton;
            int n = 0;
            int n2 = 0;
            if (this.size.equals(InventorySize.SMALL)) {
                n = 1;
                n2 = 9;
            }
            else if (this.size.equals(InventorySize.MEDIUM)) {
                n = 2;
                n2 = 9;
            }
            else if (this.size.equals(InventorySize.LARGE)) {
                n = 3;
                n2 = 9;
            }
            else if (this.size.equals(InventorySize.MAX)) {
                n = 2;
                n2 = 18;
            }
            for (int i = 0; i < n; ++i) {
                if (hasBackButton) {
                    this.inventory.setItem(n2 + i * 9, (ItemStack)Customization.getInstance().items.get("Previous"));
                }
                if (hasNextButton) {
                    this.inventory.setItem(n2 + i * 9 + 8, (ItemStack)Customization.getInstance().items.get("Next"));
                }
            }
        }
        
        public boolean expand() {
            if (this.size.equals(InventorySize.MAX)) {
                return false;
            }
            InventorySize inventorySize;
            if (this.size.equals(InventorySize.SMALL)) {
                inventorySize = InventorySize.MEDIUM;
            }
            else if (this.size.equals(InventorySize.MEDIUM)) {
                inventorySize = InventorySize.LARGE;
            }
            else {
                inventorySize = InventorySize.MAX;
            }
            final List<ItemStack> contents = this.getContents();
            this.setupCustomInventory(this.title, inventorySize, this.specialItems, this.id, this.subId);
            this.addNavigationButtons(this.hasNextButton, this.hasBackButton);
            int index = 0;
            final Iterator<ItemStack> iterator = contents.iterator();
            while (iterator.hasNext()) {
                this.inventory.setItem((int)this.slots.get(index), (ItemStack)iterator.next());
                ++index;
            }
            return true;
        }
        
        public void setItem(final int n, final ItemStack itemStack) {
            this.inventory.setItem(n, itemStack);
        }
        
        public PositionData addItem(final ItemStack itemStack, final int n, final int n2, final HashMap<Integer, ArrayList<CustomInventory>> hashMap) {
            final int emptySlot = this.getEmptySlot();
            if (emptySlot != -1) {
                this.inventory.setItem(emptySlot, itemStack);
                return new PositionData(n, n2, emptySlot);
            }
            if (this.expand()) {
                final int emptySlot2 = this.getEmptySlot();
                this.inventory.setItem(emptySlot2, itemStack);
                return new PositionData(n, n2, emptySlot2);
            }
            return this.addInventory(InventorySize.SMALL, hashMap).addItem(itemStack, n, n2 + 1, hashMap);
        }
        
        public CustomInventory addInventory(final InventorySize inventorySize, final HashMap<Integer, ArrayList<CustomInventory>> hashMap) {
            final CustomInventory e = new CustomInventory(this.title, inventorySize, this.specialItems, this.id, this.subId + 1);
            e.addNavigationButtons(false, true);
            final ArrayList value = hashMap.get(this.id);
            value.add(e);
            hashMap.put(this.id, value);
            this.addNavigationButtons(true, false);
            return e;
        }
        
        public void clear() {
            final Iterator<Integer> iterator = this.slots.iterator();
            while (iterator.hasNext()) {
                this.inventory.setItem((int)iterator.next(), new ItemStack(Material.AIR));
            }
        }
        
        public ArrayList<Integer> getEmptySlots() {
            final ArrayList list = new ArrayList();
            for (final int intValue : this.slots) {
                if (this.inventory.getItem(intValue) == null) {
                    list.add(intValue);
                }
            }
            return list;
        }
        
        public int getEmptySlot() {
            for (final int intValue : this.slots) {
                if (this.inventory.getItem(intValue) == null) {
                    return intValue;
                }
            }
            return -1;
        }
        
        public List<ItemStack> getContents() {
            final ArrayList list = new ArrayList();
            final Iterator<Integer> iterator = this.slots.iterator();
            while (iterator.hasNext()) {
                final ItemStack item = this.inventory.getItem((int)iterator.next());
                if (item != null) {
                    list.add(item);
                }
            }
            return list;
        }
        
        public void organize() {
            final ArrayList list = new ArrayList();
            for (final int intValue : this.slots) {
                final ItemStack item = this.inventory.getItem(intValue);
                if (item != null) {
                    list.add(item);
                    this.inventory.setItem(intValue, new ItemStack(Material.AIR));
                }
            }
            int index = 0;
            final Iterator iterator2 = list.iterator();
            while (iterator2.hasNext()) {
                this.inventory.setItem((int)this.slots.get(index), (ItemStack)iterator2.next());
                ++index;
            }
        }
        
        public void removeItem(final ItemStack itemStack) {
            for (final int intValue : this.slots) {
                final ItemStack item = this.inventory.getItem(intValue);
                if (item != null && item.equals((Object)itemStack)) {
                    this.removeItem(intValue);
                    break;
                }
            }
        }
        
        public void removeItem(final int n) {
            this.inventory.setItem(n, new ItemStack(Material.AIR));
            this.organize();
        }
    }
}
